Microsoft Visual Studio Solution File, Format Version 12.00
# Visual Studio 15
VisualStudioVersion = 15.0.26403.0
MinimumVisualStudioVersion = 10.0.40219.1
Project("{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}") = "vspk", "vspk\vspk.csproj", "{A12DA736-A2CA-47E2-9A20-991330B39F45}"
EndProject
Global
    GlobalSection(SolutionConfigurationPlatforms) = preSolution
        Debug|Any CPU = Debug|Any CPU
        Release|Any CPU = Release|Any CPU
    EndGlobalSection
    GlobalSection(ProjectConfigurationPlatforms) = postSolution
        {A12DA736-A2CA-47E2-9A20-991330B39F45}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
        {A12DA736-A2CA-47E2-9A20-991330B39F45}.Debug|Any CPU.Build.0 = Debug|Any CPU
        {A12DA736-A2CA-47E2-9A20-991330B39F45}.Release|Any CPU.ActiveCfg = Release|Any CPU
        {A12DA736-A2CA-47E2-9A20-991330B39F45}.Release|Any CPU.Build.0 = Release|Any CPU
    EndGlobalSection
    GlobalSection(SolutionProperties) = preSolution
        HideSolutionNode = FALSE
    EndGlobalSection
EndGlobal

