{{header}}

package {{ name }}

const (
    SDKAPIPrefix       = "{{ api_prefix }}"
    SDKAPIVersion      = {{ version }}
    SDKName            = "{{ name }}"
    SDKProductAccronym = "{{ product_accronym }}"
    SDKProductName     = "{{ product_name }}"
    SDKRootAPI         = "{{root_api}}"
)
