# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import re


def _string_clean(string):
    """
    """
    rep = {
        "IPID": "IpID",
        "VCenter": "Vcenter",
        "vCenter": "Vcenter",
        "VPort": "Vport",
        "IPv6": "Ipv6",
        "IPv4": "Ipv4",
        "DHCPv4": "Dhcpv4",
        "DHCPv6": "Dhcpv6"
    }

    rep = dict((re.escape(k), v) for k, v in rep.items())
    pattern = re.compile("|".join(list(rep.keys())))

    return pattern.sub(lambda m: rep[re.escape(m.group(0))], string)


def get_idiomatic_name(name):
    """
    """
    first_cap_re = re.compile("(.)([A-Z](?!s([A-Z])*)[a-z]+)")
    all_cap_re = re.compile("([a-z0-9])([A-Z])")

    s1 = first_cap_re.sub(r"\1_\2", _string_clean(name))

    return all_cap_re.sub(r"\1_\2", s1).lower()


def get_type_name(type_name, sub_type=None):
    """
    """
    if type_name in ("string", "enum"):
        return "str"

    if type_name == "boolean":
        return "bool"

    if type_name in ("integer", "long"):
        return "int"

    if type_name == "time":
        return "float"

    if type_name == "object":
        return "dict"

    return type_name
