<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!DOCTYPE properties SYSTEM "http://java.sun.com/dtd/properties.dtd">
<properties>
<comment>Exported from [server]</comment>
<entry key="categoryPath">Library.VSPK.Basic.{{ workflow_package }}</entry>
<entry key="name">{{ workflow_name }}</entry>
<entry key="type">Workflow</entry>
<entry key="id">{{ workflow_id }}</entry>
</properties>