<?xml version="1.0"?>
<package >
  <metadata>
    <id>net.nuagenetworks.vspk.dll</id>
    <version>{{library_version}}</version>
    <authors>nuagenetworks</authors>
    <owners>nuagenetworks</owners>
    <projectUrl>https://github.com/nuagenetworks/vspk-csharp</projectUrl>
    <requireLicenseAcceptance>false</requireLicenseAcceptance>
    <description>Nuage VSPK for .NET</description>
    <releaseNotes></releaseNotes>
    <copyright>Copyright 2017</copyright>
    <tags>nuage</tags>
    <dependencies>
      <dependency id="net.nuagenetworks.bambou.dll" version="1.1.1" />
      <dependency id="Newtonsoft.Json" version="10.0.3" />
    </dependencies>
  </metadata>
  <files>
    <file src="vspk/bin/Release/net.nuagenetworks.vspk.dll" target="lib/net452"/>
  </files>
</package>
