<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!DOCTYPE properties SYSTEM "http://java.sun.com/dtd/properties.dtd">
<properties>
<comment>135.121.118.50:8281</comment>
<entry key="vso-build-nb">3000581</entry>
<entry key="pkg-description"></entry>
<entry key="pkg-name">{{ package_name }}</entry>
<entry key="used-plugins">{{ name | upper }}#{{ version }}</entry>
<entry key="pkg-owner">CN=VMWare,OU=VMWare,OU=VMWare,O=VMWare,C=US</entry>
<entry key="vso-version">6.0.3</entry>
<entry key="pkg-id">07f9f6b25415d7b1015415e3ee7e002e</entry>
</properties>
